==> cirq_core-1.5.0.dev20250405050852-py3-none-any.whl: cirq/vis/state_histogram.py <==
# Copyright 2021 The Cirq Developers
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tool to visualize the results of a study."""

import collections
from typing import Optional, Sequence, SupportsFloat, Union

import matplotlib.pyplot as plt
import numpy as np

import cirq.study.result as result


def get_state_histogram(result: 'result.Result') -> np.ndarray:
    """Computes a state histogram from a single result with repetitions.

    Args:
        result: The trial result containing measurement results from which the
                state histogram should be computed.

    Returns:
        The state histogram (a numpy array) corresponding to the trial result.
    """
    num_qubits = sum([value.shape[1] for value in result.measurements.values()])
    states = 2**num_qubits
    values = np.zeros(states)
    # measurements is a dict of {measurement gate key:
    #                            array(repetitions, boolean result)}
    # Convert this to an array of repetitions, each with an array of booleans.
    # e.g. {q1: array([[True, True]]), q2: array([[False, False]])}
    #      --> array([[True, False], [True, False]])
    measurement_by_result = np.hstack(list(result.measurements.values()))

    for meas in measurement_by_result:
        # Convert each array of booleans to a string representation.
        # e.g. [True, False] -> [1, 0] -> '10' -> 2
        state_ind = int(''.join([str(x) for x in [int(x) for x in meas]]), 2)
        values[state_ind] += 1
    return values


def plot_state_histogram(
    data: Union['result.Result', collections.Counter, Sequence[SupportsFloat]],
    ax: Optional[plt.Axes] = None,
    *,
    tick_label: Optional[Sequence[str]] = None,
    xlabel: Optional[str] = 'qubit state',
    ylabel: Optional[str] = 'result count',
    title: Optional[str] = 'Result State Histogram',
) -> plt.Axes:
    """Plot the state histogram from either a single result with repetitions or
       a histogram computed using `result.histogram()` or a flattened histogram
       of measurement results computed using `get_state_histogram`.

    Args:
        data:   The histogram values to plot. Possible options are:
                `result.Result`: Histogram is computed using
                    `get_state_histogram` and all 2 ** num_qubits values are
                    plotted, including 0s.
                `collections.Counter`: Only (key, value) pairs present in
                    collection are plotted.
                `Sequence[SupportsFloat]`: Values in the input sequence are
                    plotted. i'th entry corresponds to height of the i'th
                    bar in histogram.
        ax:      The Axes to plot on. If not given, a new figure is created,
                 plotted on, and shown.
        tick_label: Tick labels for the histogram plot in case input is not
                    `collections.Counter`. By default, label for i'th entry
                     is |i>.
        xlabel:  Label for the x-axis.
        ylabel:  Label for the y-axis.
        title:   Title of the plot.

    Returns:
        The axis that was plotted on.
    """
    show_fig = not ax
    if ax is None:
        fig, ax = plt.subplots(1, 1)
    if isinstance(data, result.Result):
        values = get_state_histogram(data)
    elif isinstance(data, collections.Counter):
        tick_label, counts = zip(*sorted(data.items()))
        values = np.asarray(counts)
    else:
        values = np.array(data)
    if tick_label is None:
        tick_label = [str(i) for i in range(len(values))]
    ax.bar(np.arange(len(values)), values, tick_label=tick_label)
    if xlabel:
        ax.set_xlabel(xlabel)
    if ylabel:
        ax.set_ylabel(ylabel)
    if title:
        ax.set_title(title)
    if show_fig:
        fig.show()
    return ax

==> cirq_core-1.6.0.dev20250716044750-py3-none-any.whl: cirq/vis/state_histogram.py <==
# Copyright 2021 The Cirq Developers
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tool to visualize the results of a study."""

from __future__ import annotations

import collections
from typing import Sequence, SupportsFloat

import matplotlib.pyplot as plt
import numpy as np

import cirq.study.result as result


def get_state_histogram(result: result.Result) -> np.ndarray:
    """Computes a state histogram from a single result with repetitions.

    Args:
        result: The trial result containing measurement results from which the
                state histogram should be computed.

    Returns:
        The state histogram (a numpy array) corresponding to the trial result.
    """
    num_qubits = sum([value.shape[1] for value in result.measurements.values()])
    states = 2**num_qubits
    values = np.zeros(states)
    # measurements is a dict of {measurement gate key:
    #                            array(repetitions, boolean result)}
    # Convert this to an array of repetitions, each with an array of booleans.
    # e.g. {q1: array([[True, True]]), q2: array([[False, False]])}
    #      --> array([[True, False], [True, False]])
    measurement_by_result = np.hstack(list(result.measurements.values()))

    for meas in measurement_by_result:
        # Convert each array of booleans to a string representation.
        # e.g. [True, False] -> [1, 0] -> '10' -> 2
        state_ind = int(''.join([str(x) for x in [int(x) for x in meas]]), 2)
        values[state_ind] += 1
    return values


def plot_state_histogram(
    data: result.Result | collections.Counter | Sequence[SupportsFloat],
    ax: plt.Axes | None = None,
    *,
    tick_label: Sequence[str] | None = None,
    xlabel: str | None = 'qubit state',
    ylabel: str | None = 'result count',
    title: str | None = 'Result State Histogram',
) -> plt.Axes:
    """Plot the state histogram from either a single result with repetitions or
       a histogram computed using `result.histogram()` or a flattened histogram
       of measurement results computed using `get_state_histogram`.

    Args:
        data:   The histogram values to plot. Possible options are:
                `result.Result`: Histogram is computed using
                    `get_state_histogram` and all 2 ** num_qubits values are
                    plotted, including 0s.
                `collections.Counter`: Only (key, value) pairs present in
                    collection are plotted.
                `Sequence[SupportsFloat]`: Values in the input sequence are
                    plotted. i'th entry corresponds to height of the i'th
                    bar in histogram.
        ax:      The Axes to plot on. If not given, a new figure is created,
                 plotted on, and shown.
        tick_label: Tick labels for the histogram plot in case input is not
                    `collections.Counter`. By default, label for i'th entry
                     is |i>.
        xlabel:  Label for the x-axis.
        ylabel:  Label for the y-axis.
        title:   Title of the plot.

    Returns:
        The axis that was plotted on.
    """
    show_fig = not ax
    if ax is None:
        fig, ax = plt.subplots(1, 1)
    if isinstance(data, result.Result):
        values = get_state_histogram(data)
    elif isinstance(data, collections.Counter):
        tick_label, counts = zip(*sorted(data.items()))
        values = np.asarray(counts)
    else:
        values = np.array(data)
    if tick_label is None:
        tick_label = [str(i) for i in range(len(values))]
    ax.bar(np.arange(len(values)), values, tick_label=tick_label)
    if xlabel:
        ax.set_xlabel(xlabel)
    if ylabel:
        ax.set_ylabel(ylabel)
    if title:
        ax.set_title(title)
    if show_fig:
        fig.show()
    return ax
